#!/usr/bin/env python

"""
CourseBuilder 

Coursebuilder is a preprocessor tool for [pandoc](https://pandoc.org) 
to generate multi-lingual curricula documentation tables from 
structured representations as a flatfile database. Data scheme and 
actual values are kept in YAML files in order to version them with git.

"""

from argparse import ArgumentParser
import os,sys
import yaml
import pandas as pd

from tablegenerator import TableGenerator
from markdowngenerator import MarkdownGenerator
from templategenerator import TemplateGenerator
from converter import Converter

class CourseBuilder:

    @staticmethod
    def generate(args):
        if args.schema and args.meta and len(args.fields) > 0:
            
            # get actual fields
            actual_fields = []

            # use a file instead of list
            if os.path.isfile(args.fields[0]):
                with open(args.fields[0]) as ff:
                    actual_fields = yaml.load(ff,Loader=yaml.Loader)['fields']
            else:
                actual_fields = args.fields 
            
        
            # get schema
            actual_schema = None
            with open(args.schema) as f:
                actual_schema = yaml.load(f,Loader=yaml.Loader)

            # iterate through meta files
            for m in args.meta:
                with open(m) as fm:
        
                    generator = Converter()
                    generator.set_schema(actual_schema)

                    meta = yaml.load(fm,Loader=yaml.Loader)

                    table_items = generator.process(meta=meta,fields=actual_fields,lang=args.lang)

                    if args.legacy:
                        MarkdownGenerator.generate_table_legacy(table_items=table_items,add_pagebreak=args.pagebreak,title_template=args.title,first_colwidth=args.maxcol)
                    else:
                        MarkdownGenerator.generate_table(table_items=table_items,add_pagebreak=args.pagebreak,title_template=args.title,first_colwidth=args.maxcol)


                        # MarkdownGenerator.generate(table_items,pagebreak=args.pagebreak,title=args.title,header_level=args.level)

    @staticmethod
    def run():
        
        # arguments
        parser = ArgumentParser(description='versatile curricula generator')

        parser.add_argument('-m','--meta',action="extend", nargs="+", type=str,help="course description(s) as YAML file(s)")
        parser.add_argument('-l','--lang',help="Language to parse from meta file (use de or en)",default='de')
        parser.add_argument('-f','--fields',help="Fields to be used, the table will be build accordingly",action="extend", nargs="+", type=str)
        parser.add_argument('-s','--schema',help="using provided schema")
        parser.add_argument('-p','--pagebreak',action="store_true",help="add a pagebreak after each module")
        parser.add_argument('--title',type=str,default=None,help="template for title - use curly brackets (i.e. {}) to mark where the title string is inserted")
        parser.add_argument('-b','--book',type=str,help="process a whole curriculum book with sections")
        parser.add_argument('--level',type=int,default=1,help="level of header tags")
        parser.add_argument('--table-gen',type=str,default=None,help='runs table generator')
        parser.add_argument('--template',type=str,default=None,help='defines a template to be used with fields')
        parser.add_argument('-o','--out',type=str,default=None,help='set the output type')
        parser.add_argument('--legacy',action="store_true",help="use legacy generator mode for compatibility")
        

        parser.add_argument('--maxcol',type=int,default=28,help='maximum size of left column')

        # get arguments
        args = parser.parse_args()
        
        if args.table_gen:

            tg = TableGenerator()            
            tg.generate_table(args.table_gen)

            return

        # book mode with predefined setting from a book file
        if args.book and args.schema:
            
            generator = Converter()

            with open(args.schema) as sf:
                generator.set_schema(yaml.load(sf,Loader=yaml.Loader))

            with open(args.book) as bf:

                actual_fields = []

                book = yaml.load(bf,Loader=yaml.Loader)
                book_path = os.path.abspath(args.book)

                for bi in book['book']:
                    if 'fields' in bi:
                        actual_fields = bi['fields']
                    if 'sections' in bi:
                        for section in bi['sections']:
                            if 'text' in section:
                                print(section['text'][args.lang])
                            if 'modules' in section:

                                args.fields = actual_fields
                                
                                # expand filenames to be relative to the book
                                args.meta = [os.path.join(os.path.dirname(book_path),mod_path) for mod_path in section['modules']]
                                
                                CourseBuilder.generate(args=args)

        # verbose command line mode
        elif args.schema:
            CourseBuilder.generate(args=args)
        else:
            parser.print_help()

if __name__ == '__main__':
    CourseBuilder.run()
